
all:	msg swaplns messages argarray longwrds \
	getline inftest fstabplus compare arrayref rs fsrs rand \
	fsbs negexp asgext anchgsub splitargv awkpath nfset reparse

gawk.extensions:	fieldwidths ignorecase posix manyfiles igncfs

bigtest:	all pound-bang gawk.extensions

extra:	regtest

pound-bang::
	cp ../gawk /tmp && chmod +x pound-bang && ./pound-bang pound-bang >tmp
	rm -f /tmp/gawk
	cmp pound-bang.good tmp && rm -f tmp

msg::
	@echo 'Any output from "cmp" is bad news, although some differences'
	@echo 'in floating point values are probably benign -- in particular,'
	@echo 'some systems may omit a leading zero and the floating point'
	@echo 'precision may lead to slightly different output in a few cases.'

swaplns::
	gawk -f swaplns.awk data 
	
messages::
	gawk -f messages.awk 
	
argarray::
	gawk -f argarray.awk 
	
fstabplus::
	echo '1		2' | gawk -f fstabplus 
	
fsrs::
	gawk -f fsrs.awk fsrs.in 
	
igncfs::
	gawk -f igncfs.awk igncfs.in 
	
longwrds::
	gawk -f longwrds.awk manpage  
	
fieldwidths::
	echo '123456789' | gawk -v FIELDWIDTHS="2 3 4" '{print $$2}' 
	
ignorecase::
	echo xYz | gawk -v IGNORECASE=1 '{sub(/y/, ""); print}' 
	

regtest::
	@echo 'Some of the output from regtest is very system specific, do not'
	@echo 'be distressed if your output differs from that distributed.'
	@echo 'Manual inspection is called for.'
	AWK=`pwd`/../gawk ./regtest

posix::
	@echo '1:2,3 4' | ../gawk -f posix >tmp
	cmp posix.good tmp && rm -f tmp

manyfiles::
	@mkdir junk
	@../gawk 'BEGIN { for (i = 1; i <= 100; i++) print i, i}' >tmp
	@../gawk -f manyfiles.awk tmp tmp
	@echo -n "This number better be 1 ->"
	@wc -l junk/* | ../gawk '$$1 != 2' | wc -l
	@rm -rf junk tmp

compare::
	@../gawk -f compare.awk 0 1 compare.in >tmp
	cmp compare.good tmp && rm -f tmp

arrayref::
	@../gawk -f arrayref >tmp
	cmp arrayref.good tmp && rm -f tmp

rs::
	@../gawk -v RS="" '{print $$1, $$2}' rs.data >tmp
	cmp rs.good tmp && rm -f tmp

fsbs::
	@../gawk -v FS='\' '{ print $$1, $$2 }' fsbs.in >tmp
	cmp fsbs.good tmp && rm -f tmp

inftest::
	@../gawk -f inftest.awk >tmp
	cmp inftest.good tmp && rm -f tmp

getline::
	@../gawk -f getline.awk getline.awk getline.awk >tmp
	cmp getline.good tmp && rm -f tmp

rand::
	@echo The following line should just be 19 random numbers between 1 and 100
	@../gawk -f rand.awk

negexp::
	@../gawk 'BEGIN {a = -2; print 10^a }' >tmp
	cmp negexp.good tmp && rm -f tmp

asgext::
	@../gawk -f asgext.awk asgext.in >tmp
	cmp asgext.good tmp && rm -f tmp

anchgsub::
	@../gawk -f anchgsub.awk anchgsub.in >tmp
	cmp anchgsub.good tmp && rm -f tmp

splitargv::
	@../gawk -f splitargv.awk splitargv.in >tmp
	cmp splitargv.good tmp && rm -f tmp

awkpath::
	@AWKPATH=".:lib" ../gawk -f awkpath.awk >tmp
	cmp awkpath.good tmp && rm -f tmp

nfset::
	@../gawk -f nfset.awk nfset.in >tmp
	cmp nfset.good tmp && rm -f tmp

reparse::
	@../gawk -f reparse.awk reparse.in >tmp
	cmp reparse.good tmp && rm -f tmp
